SystemOrganization addCategory: 'Seaside-REST-Core'!
SystemOrganization addCategory: 'Seaside-REST-Pharo'!
SystemOrganization addCategory: 'Seaside-REST-Route'!
SystemOrganization addCategory: 'Seaside-REST-Tests'!

WARequestHandler subclass: #WARestfulHandler
	instanceVariableNames: 'routes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core'!

WARestfulHandler subclass: #WAPragmaBasedRestfulHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Pharo'!

!WAPragmaBasedRestfulHandler commentStamp: '<historical>' prior: 0!
A WAPragmaBasedRestfulHandler is a WARestfulHandler where routes are automatically created by looking at specific pragmas in the instance side methods.

One and only one of the following pragmas has to be present for a method to get picked up:
<COPY>
<DELETE>
<GET>
<MOVE>
<POST>
<PUT>
it will then by invoked by a matching HTTP request.

Optionally a <Path:> pragma may be present to customize the path to be used. The argument is an URL that can contain parameters and placeholders. Placeholders can either be argument indices or argument names.

Examples
<Path: '/{databaseName}/_all_docs'>
<Path: '/{1}/_all_docs'>!

!WAPragmaBasedRestfulHandler methodsFor: 'private' stamp: 'pmm 6/27/2010 12:11'!
createRoutes
	^ WAPragmaBasedRouteBuilder createRoutesFor: self class! !

WAPragmaBasedRestfulHandler subclass: #WAPragmaStubHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WAPragmaStubHandler methodsFor: 'public-database' stamp: 'pmm 6/13/2010 09:58'!
compactDatabase: databaseName
	<POST>
	<Path: '/{databaseName}/_compact'>! !

!WAPragmaStubHandler methodsFor: 'public-documents' stamp: 'pmm 6/13/2010 09:59'!
createBulkDocumentsInDatabase: databaseName
	<POST>
	<Path: '/{databaseName}/_bulk_docs'>! !

!WAPragmaStubHandler methodsFor: 'public-database' stamp: 'pmm 5/30/2010 20:10'!
createDatabase: databaseName
	<PUT>! !

!WAPragmaStubHandler methodsFor: 'public-documents' stamp: 'pmm 5/30/2010 20:13'!
createNewDocumentInDatabase: databaseName
	<POST>! !

!WAPragmaStubHandler methodsFor: 'public-documents' stamp: 'pmm 6/13/2010 09:59'!
createOrUpdateDocument: documentId inDatabase: databaseName
	<PUT>
	<Path: '/{databaseName}/{documentId}'>! !

!WAPragmaStubHandler methodsFor: 'public-database' stamp: 'pmm 5/30/2010 20:10'!
deleteDatabase: databaseName
	<DELETE>! !

!WAPragmaStubHandler methodsFor: 'public-documents' stamp: 'pmm 6/13/2010 09:59'!
deleteDocument: documentId inDatabase: databaseName
	<DELETE>
	<Path: '/{databaseName}/{documentId}'>! !

!WAPragmaStubHandler methodsFor: 'public-meta' stamp: 'pmm 5/30/2010 20:00'!
getAllDatabases
	<GET>
	<Path: '/_all_dbs'>! !

!WAPragmaStubHandler methodsFor: 'public-documents' stamp: 'pmm 6/13/2010 10:00'!
getAllDocuments: databaseName
	<GET>
	<Path: '/{databaseName}/_all_docs'>! !

!WAPragmaStubHandler methodsFor: 'public-meta' stamp: 'pmm 5/30/2010 20:02'!
getConfig
	<GET>
	<Path: '/_config'>! !

!WAPragmaStubHandler methodsFor: 'public-database' stamp: 'pmm 5/30/2010 20:11'!
getDatabase: databaseName
	<GET>! !

!WAPragmaStubHandler methodsFor: 'public-documents' stamp: 'pmm 6/13/2010 10:00'!
getDocument: documentId ofDatabase: databaseName
	<GET>
	<Path: '/{databaseName}/{documentId}'>! !

!WAPragmaStubHandler methodsFor: 'public-meta' stamp: 'pmm 5/30/2010 20:00'!
getInfo
	<GET>! !

!WAPragmaStubHandler methodsFor: 'public-meta' stamp: 'pmm 5/30/2010 20:02'!
getStats
	<GET>
	<Path: '/_stats'>! !

!WAPragmaStubHandler methodsFor: 'public-meta' stamp: 'pmm 6/13/2010 10:01'!
getUuids: numberOfUuids
	<GET>
	<Path: '/_uuids?count={numberOfUuids}'>! !

!WAPragmaStubHandler methodsFor: 'public-documents' stamp: 'pmm 6/13/2010 10:05'!
getView: viewName inDesign: designName inDatabase: databaseName
	<GET>
	<Path: '/{databaseName}/_design/{designName}/_view/{viewName}'>! !

!WAPragmaStubHandler methodsFor: 'public-documents' stamp: 'pmm 6/13/2010 10:00'!
queryDatabase: databaseName
	<POST>
	<Path: '/{databaseName}/_temp_view'>! !

!WAPragmaStubHandler methodsFor: 'public-meta' stamp: 'pmm 5/30/2010 20:09'!
replicate
	<POST>
	<Path: '/_replicate'>! !

!WARestfulHandler methodsFor: 'private' stamp: 'pmm 9/1/2009 17:45'!
createRoutes
	self subclassResponsibility! !

!WARestfulHandler methodsFor: 'request-handling' stamp: 'pmm 6/27/2010 12:04'!
handleFiltered: aRequestContext
	| result |
	result := self
		routeForMethod: aRequestContext request method
		consumer: aRequestContext consumer
		parameters: aRequestContext request url copy
		ifAbsent: [ self noRouteFound: aRequestContext ].
	result sendMessageTo: self! !

!WARestfulHandler methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 22:56'!
initialize
	super initialize.
	self initializeRoutes! !

!WARestfulHandler methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 22:56'!
initializeRoutes
	routes := WAByMethodRoutes routes: self createRoutes! !

!WARestfulHandler methodsFor: 'private' stamp: 'pmm 6/27/2010 12:09'!
isImplemented: aString
	^ WARoute httpMethodNames includes: aString! !

!WARestfulHandler methodsFor: 'request-handling' stamp: 'pmm 6/27/2010 12:08'!
noRouteFound: aRequestContext
	super handleFiltered: aRequestContext! !

!WARestfulHandler methodsFor: 'private' stamp: 'pmm 9/8/2009 22:24'!
routeForMethod: aString consumer: aPathConsumer parameters: aDictionary ifAbsent: aNiliadicBlock
	^ routes
		routeForMethod: aString
		consumer: aPathConsumer
		parameters: aDictionary
		ifAbsent: aNiliadicBlock! !

WARestfulHandler subclass: #WAStubHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WAStubHandler methodsFor: 'private' stamp: 'pmm 9/6/2009 23:51'!
createRoutes
	^OrderedCollection new
		"GET"
		add: (WARoute get: #listDatabases);
		add: (WARoute post: #createDatabase:);
		add: (WARoute post: #createDatabase:document:);
		add: (WARoute get: #getDatabase:document:);
		add: (WARoute delete: #deleteDatabase:document:);
		yourself! !

TestCase subclass: #WAByMethodRoutesTest
	instanceVariableNames: 'routes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WAByMethodRoutesTest methodsFor: 'private' stamp: 'pmm 9/9/2009 19:23'!
assertMethod: aMethodString selector: aSymbol path: aFirstCollectionOfStrings arguments: aSecondCollectionOfStrings
	self
		assertMethod: aMethodString
		selector: aSymbol
		path: aFirstCollectionOfStrings
		parameters: Dictionary new
		arguments: aSecondCollectionOfStrings! !

!WAByMethodRoutesTest methodsFor: 'private' stamp: 'pmm 9/9/2009 19:26'!
assertMethod: aMethodString selector: aSymbol path: aFirstCollectionOfStrings parameters: aDictionary arguments: aSecondCollectionOfStrings
	| result stub |
	result := routes
		routeForMethod: aMethodString
		consumer: (WAPathConsumer path: aFirstCollectionOfStrings asOrderedCollection)
		parameters: aDictionary
		ifAbsent: [ self assert: false ].
	stub := WAByMethodRoutesStub new.
	result sendMessageTo: stub.
	self assert: stub selector = aSymbol.
	self assert: stub arguments = aSecondCollectionOfStrings! !

!WAByMethodRoutesTest methodsFor: 'private' stamp: 'pmm 9/9/2009 19:15'!
createRoutes
	^OrderedCollection new
		"Meta"
		add: (WARoute get: #getInfo);
		add: (WARoute get: '/_all_dbs' selector: #getAllDatabases);
		add: (WARoute get: '/_config' selector: #getConfig);
		add: (WARoute get: '/_stats' selector: #getStats);
		add: (WARoute get: '/_uuids?count={1}' selector: #getUuids:);
		add: (WARoute post: '/_replicate' selector: #replicate);
		
		"Database"
		add: (WARoute post: '/{1}/_compact' selector: #compactDatabase:);
		add: (WARoute put: #createDatabase:);
		add: (WARoute delete: #deleteDatabase:);
		add: (WARoute get: #getDatabase:);
		
		"Documents"
		add: (WARoute get: '/{1}/_all_docs' selector: #getAllDocuments:);
		add: (WARoute get: '/{2}/{1}' selector: #getDocument:ofDatabase:);
		add: (WARoute post: #createNewDocumentInDatabase:);
		add: (WARoute put: '/{2}/{1}' selector: #createOrUpdateDocument:inDatabase:);
		add: (WARoute delete: '/{2}/{1}' selector: #deleteDocument:inDatabase:);
		add: (WARoute post: '/{1}/_bulk_docs' selector: #createBulkDocumentsInDatabase:);
		add: (WARoute post: '/{1}/_temp_view' selector: #queryDatabase:);
		add: (WARoute get: '/{3}/_design/{2}/_view/{1}' selector: #getView:inDesign:inDatabase:);
		yourself! !

!WAByMethodRoutesTest methodsFor: 'running' stamp: 'pmm 9/9/2009 19:05'!
setUp
	super setUp.
	routes := WAByMethodRoutes routes: self createRoutes! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:17'!
testCompactDatabase
	self
		assertMethod: 'POST'
		selector: #compactDatabase:
		path: #('credit_cards' '_compact')
		arguments: #('credit_cards')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:21'!
testCreateBulkDocumentsInDatabase
	self
		assertMethod: 'POST'
		selector: #createBulkDocumentsInDatabase:
		path: #('credit_cards' '_bulk_docs')
		arguments: #('credit_cards')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:17'!
testCreateDatabase
	self
		assertMethod: 'PUT'
		selector: #createDatabase:
		path: #('credit_cards')
		arguments: #('credit_cards')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:19'!
testCreateNewDocumentInDatabase
	self
		assertMethod: 'POST'
		selector: #createNewDocumentInDatabase:
		path: #('credit_cards')
		arguments: #('credit_cards')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:20'!
testCreateOrUpdateDocumentinDatabase
	self
		assertMethod: 'PUT'
		selector: #createOrUpdateDocument:inDatabase:
		path: #('credit_cards' '1111-2222-3333-4444')
		arguments: #('1111-2222-3333-4444' 'credit_cards')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:17'!
testDeleteDatabase
	self
		assertMethod: 'DELETE'
		selector: #deleteDatabase:
		path: #('credit_cards')
		arguments: #('credit_cards')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:20'!
testDeleteDocumentInDatabase
	self
		assertMethod: 'DELETE'
		selector: #deleteDocument:inDatabase:
		path: #('credit_cards' '1111-2222-3333-4444')
		arguments: #('1111-2222-3333-4444' 'credit_cards')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:10'!
testGetAllDatabases
	self
		assertMethod: 'GET'
		selector: #getAllDatabases
		path: #('_all_dbs')
		arguments: #()! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:11'!
testGetConfig
	self
		assertMethod: 'GET'
		selector: #getConfig
		path: #('_config')
		arguments: #()! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:13'!
testGetDatabase
	self
		assertMethod: 'GET'
		selector: #getDatabase:
		path: #('credit_cards')
		arguments: #('credit_cards')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:16'!
testGetDocumentOfDatabase
	self
		assertMethod: 'GET'
		selector: #getDocument:ofDatabase:
		path: #('credit_cards' '1111-2222-3333-4444')
		arguments: #('1111-2222-3333-4444' 'credit_cards')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:10'!
testGetInfo
	self
		assertMethod: 'GET'
		selector: #getInfo
		path: #()
		arguments: #()! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:11'!
testGetStats
	self
		assertMethod: 'GET'
		selector: #getStats
		path: #('_stats')
		arguments: #()! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:25'!
testGetUuids
	self
		assertMethod: 'GET'
		selector: #getUuids:
		path: #('_uuids')
		parameters: (Dictionary new
			at: 'count' put: '100';
			yourself)
		arguments: #('100')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:23'!
testGetViewInDesignInDatabase
	self
		assertMethod: 'GET'
		selector: #getView:inDesign:inDatabase:
		path: #('credit_cards' '_design' 'fraud' '_view' 'recent')
		arguments: #('recent' 'fraud' 'credit_cards')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:21'!
testQueryDatabase
	self
		assertMethod: 'POST'
		selector: #queryDatabase:
		path: #('credit_cards' '_temp_view')
		arguments: #('credit_cards')! !

!WAByMethodRoutesTest methodsFor: 'testing' stamp: 'pmm 9/9/2009 19:16'!
testReplicate
	self
		assertMethod: 'POST'
		selector: #replicate
		path: #('_replicate')
		arguments: #()! !

TestCase subclass: #WAComplexRouteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WAComplexRouteTest methodsFor: 'testing' stamp: 'pmm 9/6/2009 23:30'!
testInstantiation
	WAUrl absolute: '/{3}/_all_docs?startkey={1}&limit={2}'! !

!WAComplexRouteTest methodsFor: 'testing' stamp: 'pmm 9/8/2009 17:20'!
testParameterMatchingMixedParameters
	| route parameters |
	route := WARoute get: '/{1}/_all_docs?rev={2}&uuids=true' selector: #a:b:.
	self deny: (route matchesParameters: Dictionary new).
	parameters := Dictionary new at: 'rev' put: '5'; yourself.
	self deny: (route matchesParameters: parameters).
	parameters at: 'uuids' put: 'true'.
	self assert: (route matchesParameters: parameters).
	parameters at: 'phpsessionid' put: '32432'.
	self deny: (route matchesParameters: Dictionary new).
	parameters removeKey: 'phpsessionid'.
	self assert: (route matchesParameters: parameters).
	parameters at: 'uuids' put: 'false'.
	self deny: (route matchesParameters: parameters).! !

!WAComplexRouteTest methodsFor: 'testing' stamp: 'pmm 9/8/2009 17:17'!
testParameterMatchingNoParameters
	| route parameters |
	route := WARoute get: '/{1}/_all_docs' selector: #a:.
	self assert: (route matchesParameters: Dictionary new).
	parameters := Dictionary new at: 'key' put: 'value'; yourself.
	self deny: (route matchesParameters: parameters)! !

!WAComplexRouteTest methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:54'!
testPathMatching
	| route |
	route := WARoute get: '/{1}/_all_docs' selector: #a:.
	self deny: (route matchesPath: #('something')).
	self assert: (route matchesPath: #('something' '_all_docs')).
	self deny: (route matchesPath: #('something' 'else')).
	self deny: (route matchesPath: #('something' '_all_docs' 'more')).
	self assert: (route priority = 4).! !

TestCase subclass: #WAExactParameterMatchTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WAExactParameterMatchTest methodsFor: 'testing' stamp: 'pmm 6/13/2010 09:54'!
testInstantiation
	| match |
	match := WAParameterMatch name: 'rev' value: '58692' arguments: #().
	self assert: (match isKindOf: WAExactParameterMatch)! !

!WAExactParameterMatchTest methodsFor: 'testing' stamp: 'pmm 6/13/2010 09:54'!
testMatches
	| match |
	match := WAParameterMatch name: 'rev' value: '58692' arguments: #().
	self assert: (match matchesName: 'rev' value: '58692').
	self deny: (match matchesName: 'rev' value: 58692).
	self deny: (match matchesName: 'rev' value: 'a').
	self deny: (match matchesName: 'revision' value: '58692').! !

TestCase subclass: #WAExactPathMatchTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WAExactPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:52'!
testInstantiation
	| match |
	match := WAPPathMatch fromString: 'path' index: 1 arguments: #().
	self assert: (match isKindOf: WAExactPathMatch).
	self assert: match priority = 3.! !

!WAExactPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/13/2010 09:31'!
testMatches
	| match |
	match := WAPPathMatch fromString: 'path' index: 1 arguments: #().
	self assert: (match matches: 'path').
	self deny: (match matches: 'poth')! !

TestCase subclass: #WAPlaceholderParameterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WAPlaceholderParameterTest methodsFor: 'testing' stamp: 'pmm 6/13/2010 09:57'!
testInstantiation
	| match |
	match := WAParameterMatch name: 'rev' value: '{1}' arguments: #().
	self assert: (match isKindOf: WAPlaceholderParameterMatch).
	self assert: match index = 1! !

!WAPlaceholderParameterTest methodsFor: 'testing' stamp: 'pmm 6/13/2010 09:52'!
testMatches
	| match |
	match := WAParameterMatch name: 'rev' value: '{1}' arguments: #().
	self assert: (match matchesName: 'rev' value: '58692').
	self assert: (match matchesName: 'rev' value: 'a').
	self deny: (match matchesName: 'revision' value: '58692').! !

!WAPlaceholderParameterTest methodsFor: 'testing' stamp: 'pmm 6/13/2010 09:54'!
testNamedInstantiation
	| match |
	match := WAParameterMatch name: 'rev' value: '{put}' arguments: #('post' 'put' 'get').
	self assert: (match isKindOf: WAPlaceholderParameterMatch).
	self assert: match index = 2! !

TestCase subclass: #WAPlaceholderPathMatchTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WAPlaceholderPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:48'!
testFullInstantiation
	| match |
	match := WAPPathMatch fromString: '{7}' index: 2 arguments: #().
	self assert: (match isKindOf: WAFullPlaceholderPathMatch).
	self assert: match priority = 1.
	self assert: match sourceIndex = 2.
	self assert: match targetIndex = 7.! !

!WAPlaceholderPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/30/2010 07:38'!
testFullMapArgumentsWithElementsInto
	| match result |
	match := WAPPathMatch fromString: '{1}' index: 1 arguments: #().
	result := Array new: 1.
	match mapArgumentsWithElements: #('foo') into: result.
	self assert: result first = 'foo'! !

!WAPlaceholderPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:49'!
testMatchesFull
	| match |
	match := WAPPathMatch fromString: '{7}' index: 1 arguments: #().
	self assert: (match matches: 'foo')! !

!WAPlaceholderPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:51'!
testMatchesPrefix
	| match |
	match := WAPPathMatch fromString: 'foo.{7}' index: 1 arguments: #().
	self assert: (match matches: 'foo.bar').
	self deny: (match matches: 'foo.').
	self deny: (match matches: 'fo.bar').
	self deny: (match matches: 'oo.bar').! !

!WAPlaceholderPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:50'!
testMatchesSuffix
	| match |
	match := WAPPathMatch fromString: '{7}.bar' index: 1 arguments: #().
	self assert: (match matches: 'foo.bar').
	self deny: (match matches: '.bar').
	self deny: (match matches: 'foo.barz').
	self deny: (match matches: 'foo.ba').! !

!WAPlaceholderPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:48'!
testNamedInstantiation
	| match |
	match := WAPPathMatch fromString: '{put}' index: 2 arguments: #('get' 'put' 'post').
	self assert: (match isKindOf: WAFullPlaceholderPathMatch).
	self assert: match priority = 1.
	self assert: match sourceIndex = 2.
	self assert: match targetIndex = 2.! !

!WAPlaceholderPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/29/2010 08:14'!
testPrefixInstantiation
	| match |
	match := WAPPathMatch fromString: 'foo.{787}' index: 2 arguments: #().
	self assert: (match isKindOf: WAPrefixPlaceholderPathMatch).
	self assert: match priority = 2.
	self assert: match sourceIndex = 2.
	self assert: match targetIndex = 787.! !

!WAPlaceholderPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/30/2010 07:38'!
testPrefixMapArgumentsWithElementsInto
	| match result |
	match := WAPPathMatch fromString: 'foo.{1}' index: 1 arguments: #().
	result := Array new: 1.
	match mapArgumentsWithElements: #('foo.bar') into: result.
	self assert: result first = 'bar'! !

!WAPlaceholderPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/29/2010 08:14'!
testSuffixInstantiation
	| match |
	match := WAPPathMatch fromString: '{787}.bar' index: 2 arguments: #().
	self assert: (match isKindOf: WASuffixPlaceholderPathMatch).
	self assert: match priority = 2.
	self assert: match sourceIndex = 2.
	self assert: match targetIndex = 787.! !

!WAPlaceholderPathMatchTest methodsFor: 'testing' stamp: 'pmm 6/30/2010 07:38'!
testSuffixMapArgumentsWithElementsInto
	| match result |
	match := WAPPathMatch fromString: '{1}.bar' index: 1 arguments: #().
	result := Array new: 1.
	match mapArgumentsWithElements: #('foo.bar') into: result.
	self assert: result first = 'foo'! !

TestCase subclass: #WAPragmaBasedRestfulHandlerTest
	instanceVariableNames: 'handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WAPragmaBasedRestfulHandlerTest methodsFor: 'running' stamp: 'pmm 5/31/2010 20:32'!
setUp
	handler := WAPragmaStubHandler new! !

!WAPragmaBasedRestfulHandlerTest methodsFor: 'testing' stamp: 'pmm 5/31/2010 20:32'!
testCreateRoutes
	self assert: handler notNil! !

TestCase subclass: #WARouteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WARouteTest methodsFor: 'testing' stamp: 'pmm 9/8/2009 22:26'!
testCount
	| route |
	route := WARoute get: #listDatabases.
	self assert: route pathElementCount isZero.
	
	route := WARoute get: #getDatabase:.
	self assert: route pathElementCount = 1.
	
	route := WARoute get: #getDatabase:document:.
	self assert: route pathElementCount = 2! !

!WARouteTest methodsFor: 'testing' stamp: 'pmm 9/8/2009 22:19'!
testMethod
	| route |
	route := WARoute get: #listDatabases.
	self assert: route method = 'GET'.
	
	route := WARoute post: #createDatabase:document:.
	self assert: route method = 'POST'.
	
	route := WARoute copy: #copyDatabase:document:.
	self assert: route method = 'COPY'! !

!WARouteTest methodsFor: 'testing' stamp: 'pmm 9/8/2009 22:20'!
testSendSelectorWithTo
	| route victim result |
	victim := WARouteVictimMock new.
	route := WARoute get: #nothing.
	result := route sendSelectorWith: Array new to: victim.
	self assert: result = Array new.
	
	route := WARoute get: #first:.
	result := route sendSelectorWith: #('terrorists') to: victim.
	self assert: result = #('terrorists').
	
	route := WARoute get: #first:second:.
	result := route sendSelectorWith: #('terrorists' 'marschall') to: victim.
	self assert: result = #('terrorists' 'marschall').! !

TestCase subclass: #WASimpleRouteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WASimpleRouteTest methodsFor: 'testing' stamp: 'pmm 9/8/2009 17:17'!
testParameterMatching
	| route parameters |
	route := WARoute get: #a:.
	self assert: (route matchesParameters: Dictionary new).
	parameters := Dictionary new at: 'key' put: 'value'; yourself.
	self deny: (route matchesParameters: parameters)! !

!WASimpleRouteTest methodsFor: 'testing' stamp: 'pmm 9/8/2009 13:01'!
testPathMatching
	| route |
	route := WARoute get: #a:.
	self deny: (route matchesPath: Array new).
	self assert: (route matchesPath: #('something')).
	self deny: (route matchesPath: #('something' 'else')).
	self assert: (route priority = 1).
	
	route := WARoute get: #a:b:.
	self deny: (route matchesPath: Array new).
	self deny: (route matchesPath: #('something')).
	self assert: (route matchesPath: #('something' 'else')).
	self deny: (route matchesPath: #('something' 'else' 'more')).
	self assert: (route priority = 2)! !

WAContextTest subclass: #WARestfulHandlerTest
	instanceVariableNames: 'handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WARestfulHandlerTest methodsFor: 'testing' stamp: 'pmm 9/8/2009 22:21'!
setUp
	super setUp.
	handler := WAStubHandler new! !

!WARestfulHandlerTest methodsFor: 'testing' stamp: 'pmm 9/8/2009 22:25'!
testFindRouteForMethodConsumerIfAbsent
	| route result |
	result := handler
		routeForMethod: 'GET'
		consumer: (WAPathConsumer path: #('users' 'marschall') asOrderedCollection)
		parameters: Dictionary new
		ifAbsent: [ self assert: false ].
	route := result route.
	self assert: route method = 'GET'.
	self assert: route pathElementCount = 2.
	self assert: route selector = #getDatabase:document:.
	self assert: result elements = #('users' 'marschall')! !

WAObject subclass: #WAByMethodRoutes
	instanceVariableNames: 'routes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WAByMethodRoutes commentStamp: '<historical>' prior: 0!
A WAByMethodRoutes is a collection of routes first organized by HTTP method and then by number of URL path elements. It allows users to look up possible routes for a given HTTP request.

See also WARouteResult and WARouteContainer.

Instance Variables
	routes:		<Dictionary<String, SortedCollection<WARouteContainer>>

routes
	- the routes keyed by HTTP method, sourted by number of path elements
!

!WAByMethodRoutes class methodsFor: 'instance creation' stamp: 'pmm 9/8/2009 22:12'!
routes: aCollectionOfRoutes
	^ self basicNew initializeWithRoutes: aCollectionOfRoutes! !

!WAByMethodRoutes methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 22:33'!
initializeWithRoutes: aCollectionOfRoutes
	| routeBuilder |
	super initialize.
	routeBuilder := Dictionary new.
	aCollectionOfRoutes do: [ :each |
		| methodRoutes subRoutes |
		methodRoutes := routeBuilder
			at: each method
			ifAbsentPut: [ Dictionary new ].
		subRoutes := methodRoutes at: each pathElementCount ifAbsentPut: [ OrderedCollection new ].
		subRoutes add: each ].
	routes := Dictionary new.
	routeBuilder keysAndValuesDo: [ :method :subroutes |
		| methodRoutes |
		methodRoutes := routes
			at: method
			ifAbsentPut: [ SortedCollection sortBlock: [ :a :b |
					a pathElementCount <= b pathElementCount ] ].
		subroutes valuesDo: [ :each |
			methodRoutes add: (WARouteCountainer routes: each) ] ]! !

!WAByMethodRoutes methodsFor: 'public' stamp: 'pmm 9/9/2009 19:54'!
routeForMethod: aString consumer: aPathConsumer parameters: aDictionary ifAbsent: aNiliadicBlock
	| routesForMethod routesStream route count elements |
	routesForMethod := routes at: aString ifAbsent: [ ^ aNiliadicBlock value ].
	elements := OrderedCollection new.
	routesStream := routesForMethod readStream.
	routesStream atEnd ifTrue: [ ^ aNiliadicBlock value ].
	route := routesStream next.
	count := 0.
	[ aPathConsumer atEnd ]  whileFalse: [
		elements add: aPathConsumer next.
		count := count + 1.
		[ route pathElementCount < count ] whileTrue: [
			routesStream atEnd ifTrue: [ ^ aNiliadicBlock value ].
			route := routesStream next ] ].
	count = route pathElementCount
		ifFalse: [ ^ aNiliadicBlock value ].
	^ WARouteResult
		route: (route
			routeForElements: elements
			parameters: aDictionary
			ifAbsent: [ ^ aNiliadicBlock value ])
		elements: elements asArray
		parameters: aDictionary! !

WAObject subclass: #WAMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WAMatch class methodsFor: 'private' stamp: 'pmm 6/13/2010 10:01'!
argument: aString notFoundInd: aCollectionOfStrings
	WAArgumentNotFoundError signal: 'argument "', aString, '" was not found in ', aCollectionOfStrings greaseString! !

!WAMatch class methodsFor: 'private' stamp: 'pmm 6/13/2010 09:49'!
targetIndexOf: aString arguments: aCollectionOfStrings
	aString do: [ :each |
		each isDigit ifFalse: [
			| index |
			index := aCollectionOfStrings indexOf: aString.
			index isZero ifTrue: [
				self argument: aString notFoundInd: aCollectionOfStrings ].
			^index ] ].
	^ aString asNumber! !

WAMatch subclass: #WAParameterMatch
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

WAParameterMatch subclass: #WAExactParameterMatch
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WAExactParameterMatch class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:57'!
name: aNameString value: aValueString arguments: aCollectionOfStrings 
	^ self basicNew initalizeWithName: aNameString value: aValueString! !

!WAExactParameterMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:29'!
argumentCount
	^ 0! !

!WAExactParameterMatch methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 13:15'!
initalizeWithName: aNameString value: aValueString
	super initalizeWithName: aNameString.
	value := aValueString! !

!WAExactParameterMatch methodsFor: 'public' stamp: 'pmm 9/9/2009 07:06'!
mapArgumentsWithParameters: aDictionary into: anArray
	"ignore"! !

!WAExactParameterMatch methodsFor: 'testing' stamp: 'pmm 9/8/2009 13:08'!
matchesName: aNameString value: aValueString
	^name = aNameString
		and: [ value = aValueString ]! !

!WAParameterMatch class methodsFor: 'private' stamp: 'pmm 9/8/2009 13:20'!
isPattern: aString
	^ aString isEmpty not
		and: [ aString first = ${
		and: [ aString last = $} ] ]! !

!WAParameterMatch class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:57'!
name: aKeyString value: aValueString arguments: aCollectionOfStrings 
	^ ((self isPattern: aValueString)
		ifTrue: [ WAPlaceholderParameterMatch ]
		ifFalse: [ WAExactParameterMatch ]) name: aKeyString value: aValueString arguments: aCollectionOfStrings! !

!WAParameterMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:29'!
argumentCount
	self subclassResponsibility! !

!WAParameterMatch methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 13:15'!
initalizeWithName: aString
	super initialize.
	name := aString! !

!WAParameterMatch methodsFor: 'public' stamp: 'pmm 9/9/2009 07:05'!
mapArgumentsWithParameters: aDictionary into: anArray
	self subclassResponsibility! !

!WAParameterMatch methodsFor: 'testing' stamp: 'pmm 9/8/2009 13:06'!
matchesName: aNameString value: aValueString
	self subclassResponsibility! !

WAParameterMatch subclass: #WAPlaceholderParameterMatch
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WAPlaceholderParameterMatch class methodsFor: 'instance creation' stamp: 'pmm 9/8/2009 13:24'!
name: aString index: anInteger
	^ self basicNew initalizeWithName: aString index: anInteger! !

!WAPlaceholderParameterMatch class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:51'!
name: aNameString value: aValueString arguments: aCollectionOfStrings
	^ self
		name: aNameString
		index: (self
			targetIndexOf: (aValueString copyFrom: 2 to: aValueString size - 1)
			arguments: aCollectionOfStrings)! !

!WAPlaceholderParameterMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:30'!
argumentCount
	^ 1! !

!WAPlaceholderParameterMatch methodsFor: 'accessing' stamp: 'pmm 6/13/2010 09:53'!
index
	^ index! !

!WAPlaceholderParameterMatch methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 13:18'!
initalizeWithName: aString index: anInteger
	super initalizeWithName: aString.
	index := anInteger! !

!WAPlaceholderParameterMatch methodsFor: 'public' stamp: 'pmm 9/9/2009 07:06'!
mapArgumentsWithParameters: aDictionary into: anArray
	anArray at: index put: (aDictionary at: name)! !

!WAPlaceholderParameterMatch methodsFor: 'testing' stamp: 'pmm 9/8/2009 13:08'!
matchesName: aNameString value: aValueString
	^name = aNameString! !

WAMatch subclass: #WAPPathMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

WAPPathMatch subclass: #WAExactPathMatch
	instanceVariableNames: 'element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WAExactPathMatch class methodsFor: 'instance creation' stamp: 'pmm 9/7/2009 00:07'!
fromString: aString
	^ self basicNew inititializeWith: aString! !

!WAExactPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:30'!
fromString: aString index: anInteger arguments: aCollectionOfStrings
	^ self fromString: aString! !

!WAExactPathMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:30'!
argumentCount
	^ 0! !

!WAExactPathMatch methodsFor: 'initialize-release' stamp: 'pmm 9/7/2009 00:07'!
inititializeWith: aString
	super initialize.
	element := aString! !

!WAExactPathMatch methodsFor: 'public' stamp: 'pmm 9/10/2009 09:10'!
mapArgumentsWithElements: aCollection into: anArray
	"ignore"! !

!WAExactPathMatch methodsFor: 'testing' stamp: 'pmm 9/7/2009 00:07'!
matches: aString
	^ element = aString! !

!WAExactPathMatch methodsFor: 'accessing' stamp: 'pmm 6/26/2010 17:45'!
priority
	^ 3! !

WAPPathMatch subclass: #WAPlaceholderPathMatch
	instanceVariableNames: 'sourceIndex targetIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

WAPlaceholderPathMatch subclass: #WAFullPlaceholderPathMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WAFullPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 10:09'!
fromString: aString index: anInteger arguments: aCollectionOfStrings
	^ self
		source: anInteger
		target: (self
			targetIndexOf: (aString copyFrom: 2 to: aString size - 1)
			arguments: aCollectionOfStrings)! !

!WAFullPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 10:12'!
source: anFirstInteger target: aSecondInteger
	^ self basicNew initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger! !

!WAFullPlaceholderPathMatch methodsFor: 'public' stamp: 'pmm 6/30/2010 07:27'!
mapArgumentsWithElements: aCollection into: anArray
	anArray
		at: targetIndex
		put: (aCollection at: sourceIndex)! !

!WAFullPlaceholderPathMatch methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:45'!
matches: aString
	^ true! !

!WAFullPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 6/26/2010 17:44'!
priority
	^ 1! !

!WAPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:30'!
argumentCount
	^ 1! !

!WAPlaceholderPathMatch methodsFor: 'initialize-release' stamp: 'pmm 9/10/2009 09:18'!
initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger
	super initialize.
	sourceIndex := anFirstInteger.
	targetIndex := aSecondInteger! !

!WAPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 9/10/2009 09:05'!
sourceIndex
	^ sourceIndex! !

!WAPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 9/10/2009 09:11'!
targetIndex
	^ targetIndex! !

WAPlaceholderPathMatch subclass: #WAPrefixPlaceholderPathMatch
	instanceVariableNames: 'prefix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WAPrefixPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 10:20'!
fromString: aString index: anInteger arguments: aCollectionOfStrings
	| prefix |
	prefix := aString copyUpTo: ${.
	^ self
		source: anInteger
		target: (self
			targetIndexOf: (aString copyFrom: prefix size + 2 to: aString size - 1)
			arguments: aCollectionOfStrings)
		prefix: prefix! !

!WAPrefixPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 10:14'!
source: anFirstInteger target: aSecondInteger prefix: aString
	^ self basicNew
		initializeWithSourceIndex: anFirstInteger
		targetIndex: aSecondInteger
		prefix: aString! !

!WAPrefixPlaceholderPathMatch methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 10:14'!
initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger prefix: aString
	super initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger.
	prefix := aString! !

!WAPrefixPlaceholderPathMatch methodsFor: 'public' stamp: 'pmm 6/30/2010 08:43'!
mapArgumentsWithElements: aCollection into: anArray
	| value |
	value := aCollection at: sourceIndex.
	anArray
		at: targetIndex
		put: (value copyFrom: prefix size + 1 to: value size)! !

!WAPrefixPlaceholderPathMatch methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:46'!
matches: aString
	^ (aString size > prefix size)
		and: [ aString beginsWith: prefix ]! !

!WAPrefixPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 6/26/2010 17:44'!
priority
	^ 2! !

WAPlaceholderPathMatch subclass: #WASuffixPlaceholderPathMatch
	instanceVariableNames: 'suffix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WASuffixPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/29/2010 08:16'!
fromString: aString index: anInteger arguments: aCollectionOfStrings
	| suffix |
	suffix := aString copyAfter: $}.
	^ self
		source: anInteger
		target: (self
			targetIndexOf: (aString copyFrom: 2 to: aString size - suffix size - 1)
			arguments: aCollectionOfStrings)
		suffix: suffix! !

!WASuffixPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 10:14'!
source: anFirstInteger target: aSecondInteger suffix: aString
	^ self basicNew
		initializeWithSourceIndex: anFirstInteger
		targetIndex: aSecondInteger
		suffix: aString! !

!WASuffixPlaceholderPathMatch methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 10:14'!
initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger suffix: aString
	super initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger.
	suffix := aString! !

!WASuffixPlaceholderPathMatch methodsFor: 'public' stamp: 'pmm 6/30/2010 08:57'!
mapArgumentsWithElements: aCollection into: anArray
	| value |
	value := aCollection at: sourceIndex.
	anArray
		at: targetIndex
		put: (value copyFrom: 1 to: value size - suffix size)! !

!WASuffixPlaceholderPathMatch methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:47'!
matches: aString
	^ (aString size > suffix size)
		and: [ aString endsWith: suffix ]! !

!WASuffixPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 6/26/2010 17:44'!
priority
	^ 2! !

!WAPPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 00:28'!
fromString: aString index: anInteger arguments: aCollectionOfStrings
	^ (self matchClassFor: aString)
		fromString: aString index: anInteger arguments: aCollectionOfStrings! !

!WAPPathMatch class methodsFor: 'private' stamp: 'pmm 6/27/2010 10:18'!
matchClassFor: aString
	aString isEmpty ifTrue: [ ^ WAExactPathMatch  ].
	aString first = ${
		ifTrue: [
			| index |
			index := aString indexOf: $} startingAt: 2.
			index = aString size ifTrue: [ ^ WAFullPlaceholderPathMatch ].
			index isZero ifFalse: [ ^ WASuffixPlaceholderPathMatch ] ].
	(aString last = $}
		and: [ (aString indexOf: ${) isZero not ])
			ifTrue: [ ^ WAPrefixPlaceholderPathMatch ].
	^ WAExactPathMatch! !

!WAPPathMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:30'!
argumentCount
	self subclassResponsibility! !

!WAPPathMatch methodsFor: 'public' stamp: 'pmm 9/10/2009 09:10'!
mapArgumentsWithElements: aCollection into: anArray
	self subclassResponsibility! !

!WAPPathMatch methodsFor: 'testing' stamp: 'pmm 9/6/2009 23:57'!
matches: aString
	self subclassResponsibility! !

!WAPPathMatch methodsFor: 'accessing' stamp: 'pmm 9/6/2009 21:49'!
priority
	self subclassResponsibility! !

WAObject subclass: #WAPragmaBasedRouteBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Pharo'!

!WAPragmaBasedRouteBuilder class methodsFor: 'public' stamp: 'DaleHenrichs 07/10/2010 10:41'!
createRoutesFor: aClass
	| routes |
	routes := OrderedCollection new.
	aClass selectorsDo: [ :selector |
		| httpMethodName method |
		httpMethodName := nil.
		method := aClass >> selector.
		method pragmas do: [ :pragma |
			(WARoute httpMethodNames includes: pragma keyword) ifTrue: [
				httpMethodName := pragma keyword greaseString ] ].
		httpMethodName isNil ifFalse: [
			| path |
			path := method pragmas detect: [:pragma | pragma keyword == #Path: ] ifNone: [ nil ].
			routes add: (path isNil
				ifTrue: [
					WASimpleRoute
						method: httpMethodName
						selector: selector ]
				ifFalse: [
					| arguments |
					arguments := (method argsAndTemps copyFrom: 1 to: method numArgs) collect: [:each | each asString ].
					WAComplexRoute
						method: httpMethodName
						selector: selector
						pattern: (path argumentAt: 1)
						arguments: arguments ] ) ] ].
	^ routes! !

WAObject subclass: #WARoute
	instanceVariableNames: 'method selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

WARoute subclass: #WAComplexRoute
	instanceVariableNames: 'pathMatches parameterMatches'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WAComplexRoute class methodsFor: 'examples' stamp: 'pmm 9/6/2009 21:36'!
example
	^self
		get: '/{3}/_all_docs?startkey={1}&limit={2}'
		selector: #getAllDocumentsStartingAt:limitedTo:inDatabase:! !

!WAComplexRoute class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:40'!
method: aMethodString selector: aSymbol pattern: aPatternString
	^self
		method: aMethodString
		selector: aSymbol
		url: (WAUrl absolute: aPatternString)
		arguments: #()! !

!WAComplexRoute class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:39'!
method: aMethodString selector: aSymbol pattern: aPatternString arguments: aCollectionOfStrings
	^self
		method: aMethodString
		selector: aSymbol
		url: (WAUrl absolute: aPatternString)
		arguments: aCollectionOfStrings! !

!WAComplexRoute class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:38'!
method: aString selector: aSymbol url: aUrl arguments: aCollectionOfStrings
	^self basicNew initializeWithMethod: aString selector: aSymbol url: aUrl arguments: aCollectionOfStrings! !

!WAComplexRoute methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:32'!
argumentCount
	| count |
	count := 0.
	pathMatches do: [ :each |
		count := count + each argumentCount ].
	parameterMatches valuesDo: [ :each |
		count := count + each argumentCount ].
	^ count! !

!WAComplexRoute methodsFor: 'initialize-release' stamp: 'pmm 6/13/2010 09:51'!
initializeWithMethod: aString selector: aSymbol url: aUrl arguments: aCollectionOfStrings
	super initializeWithMethod: aString selector: aSymbol.
	pathMatches := aUrl path withIndexCollect: [ :each :index |
		WAPPathMatch fromString: each index: index arguments: aCollectionOfStrings ].
	parameterMatches := Dictionary new.
	aUrl parameters keysAndValuesDo: [ :key :value |
			parameterMatches
				at: key
				put: (WAParameterMatch name: key value: value arguments: aCollectionOfStrings) ]! !

!WAComplexRoute methodsFor: 'public' stamp: 'pmm 9/10/2009 09:10'!
mapArgumentsWithElements: anStringArray parameters: aDictionary into: anArray
	pathMatches do: [ :each |
		each mapArgumentsWithElements: anStringArray into: anArray ].
	parameterMatches valuesDo: [ :each |
		each mapArgumentsWithParameters: aDictionary into: anArray ]! !

!WAComplexRoute methodsFor: 'testing' stamp: 'pmm 9/8/2009 17:13'!
matchesParameters: aDictionary
	parameterMatches size = aDictionary size
		ifFalse: [ ^ false ].
	parameterMatches keysAndValuesDo: [ :name :match |
		| value |
		value := aDictionary at: name ifAbsent: [ ^ false ].
		(match matchesName: name value: value)
			ifFalse: [ ^ false ] ].
	^ true! !

!WAComplexRoute methodsFor: 'testing' stamp: 'pmm 9/7/2009 00:12'!
matchesPath: aCollectionOfStrings
	pathMatches size = aCollectionOfStrings size
		ifFalse: [ ^ false ].
	pathMatches with: aCollectionOfStrings do: [ :match :each |
		(match matches: each)
			ifFalse: [ ^ false ] ].
	^ true! !

!WAComplexRoute methodsFor: 'accessing' stamp: 'pmm 9/6/2009 22:09'!
pathElementCount
	^ pathMatches size! !

!WAComplexRoute methodsFor: 'accessing' stamp: 'pmm 9/7/2009 00:27'!
priority
	^ pathMatches inject: 0 into: [ :sum :each |
		sum + each priority ]! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
copy: aSymbol
	^ WASimpleRoute method: 'COPY' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
delete: aSymbol
	^ WASimpleRoute method: 'DELETE' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/9/2009 07:24'!
delete: aString selector: aSymbol
	^ WAComplexRoute
		method: 'DELETE'
		selector: aSymbol
		pattern: aString! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
get: aSymbol
	^ WASimpleRoute method: 'GET' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/7/2009 00:14'!
get: aString selector: aSymbol
	^ WAComplexRoute
		method: 'GET'
		selector: aSymbol
		pattern: aString! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 5/30/2010 20:25'!
httpMethodNames
	^ #(
		COPY
		DELETE
		GET
		MOVE
		POST
		PUT
	)! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
move: aSymbol
	^ WASimpleRoute method: 'MOVE' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
post: aSymbol
	^ WASimpleRoute method: 'POST' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/9/2009 07:23'!
post: aString selector: aSymbol
	^ WAComplexRoute
		method: 'POST'
		selector: aSymbol
		pattern: aString! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
put: aSymbol
	^ WASimpleRoute method: 'PUT' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/9/2009 07:23'!
put: aString selector: aSymbol
	^ WAComplexRoute
		method: 'PUT'
		selector: aSymbol
		pattern: aString! !

!WARoute methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:29'!
argumentCount
	self subclassResponsibility! !

!WARoute methodsFor: 'initialize-release' stamp: 'pmm 9/6/2009 22:06'!
initializeWithMethod: aString selector: aSymbol
	super initialize.
	method := aString.
	selector := aSymbol! !

!WARoute methodsFor: 'public' stamp: 'pmm 9/9/2009 07:00'!
mapArgumentsWithElements: anStringArray parameters: aDictionary into: anArray
	self subclassResponsibility! !

!WARoute methodsFor: 'testing' stamp: 'pmm 9/8/2009 17:13'!
matchesParameters: aDictionary
	self subclassResponsibility! !

!WARoute methodsFor: 'testing' stamp: 'pmm 9/7/2009 00:28'!
matchesPath: aCollectionOfStrings
	self subclassResponsibility! !

!WARoute methodsFor: 'accessing' stamp: 'pmm 9/6/2009 22:05'!
method
	^ method! !

!WARoute methodsFor: 'accessing' stamp: 'pmm 9/6/2009 22:07'!
pathElementCount
	self subclassResponsibility! !

!WARoute methodsFor: 'accessing' stamp: 'pmm 9/7/2009 00:25'!
priority
	self subclassResponsibility! !

!WARoute methodsFor: 'accessing' stamp: 'pmm 9/6/2009 22:06'!
selector
	^ selector! !

!WARoute methodsFor: 'public' stamp: 'pmm 9/9/2009 19:27'!
sendSelectorWith: anArrayOfArguments to: anObject
	^anObject perform: selector withArguments: anArrayOfArguments! !

WARoute subclass: #WASimpleRoute
	instanceVariableNames: 'count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WASimpleRoute class methodsFor: 'instance creation' stamp: 'pmm 9/1/2009 17:43'!
method: aString count: anInteger selector: aSymbol
	^ self basicNew initializeWithMethod: aString count: anInteger selector: aSymbol! !

!WASimpleRoute class methodsFor: 'instance creation' stamp: 'pmm 9/1/2009 17:54'!
method: aString selector: aSymbol
	^ self basicNew
		initializeWithMethod: aString
		count: (aSymbol count: [ :each | each = $: ])
		selector: aSymbol! !

!WASimpleRoute methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:29'!
argumentCount
	^ self pathElementCount! !

!WASimpleRoute methodsFor: 'initialize-release' stamp: 'pmm 9/6/2009 22:07'!
initializeWithMethod: aString count: anInteger selector: aSymbol
	super initializeWithMethod: aString selector: aSymbol.
	count := anInteger! !

!WASimpleRoute methodsFor: 'public' stamp: 'pmm 9/9/2009 07:01'!
mapArgumentsWithElements: anStringArray parameters: aDictionary into: anArray
	1 to: self argumentCount do: [ :index |
		anArray at: index put: (anStringArray at: index) ]! !

!WASimpleRoute methodsFor: 'testing' stamp: 'pmm 9/8/2009 17:13'!
matchesParameters: aDictionary
	^ aDictionary isEmpty! !

!WASimpleRoute methodsFor: 'testing' stamp: 'pmm 9/8/2009 13:02'!
matchesPath: aCollectionOfStrings
	^ self pathElementCount = aCollectionOfStrings size! !

!WASimpleRoute methodsFor: 'accessing' stamp: 'pmm 9/6/2009 21:55'!
pathElementCount
	^ count! !

!WASimpleRoute methodsFor: 'accessing' stamp: 'pmm 9/7/2009 00:25'!
priority
	^ self pathElementCount! !

WAObject subclass: #WARouteCountainer
	instanceVariableNames: 'pathElementCount routes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WARouteCountainer commentStamp: '<historical>' prior: 0!
A WARouteCountainer is a collection of routes that all accept the same number of path elements ordered by priority.

Instance Variables
	pathElementCount:		<Integer>
	routes:		<SortedCollection<WARoute>>

pathElementCount
	- the number of path elements that any of the routes in this container accepts

routes
	- the routes ordered by priority
!

!WARouteCountainer class methodsFor: 'instance creation' stamp: 'pmm 9/8/2009 21:49'!
pathElementCount: anInteger
	^ self basicNew initializeWith: anInteger! !

!WARouteCountainer class methodsFor: 'instance creation' stamp: 'pmm 9/8/2009 22:10'!
routes: aCollectionOfRoutes
	^ (self pathElementCount: aCollectionOfRoutes anyOne pathElementCount)
		addRoutes: aCollectionOfRoutes;
		yourself! !

!WARouteCountainer methodsFor: 'public' stamp: 'pmm 9/8/2009 17:24'!
addRoute: aRoute
	^ routes add: aRoute! !

!WARouteCountainer methodsFor: 'public' stamp: 'pmm 9/8/2009 22:10'!
addRoutes: aCollectionOfRoutes
	aCollectionOfRoutes do: [ :each |
		self addRoute: each ]! !

!WARouteCountainer methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 17:24'!
initializeWith: anInteger
	super initialize.
	pathElementCount := anInteger.
	routes := SortedCollection sortBlock: [ :a :b |
		a priority > b priority ]! !

!WARouteCountainer methodsFor: 'public' stamp: 'pmm 9/8/2009 17:26'!
pathElementCount
	^ pathElementCount! !

!WARouteCountainer methodsFor: 'public' stamp: 'pmm 9/9/2009 19:53'!
routeForElements: aCollection parameters: aDictionary ifAbsent: aNiliadicBlock
	^ routes
		detect: [ :each |
			(each matchesPath: aCollection)
				and: [ each matchesParameters: aDictionary ] ]
		ifNone: aNiliadicBlock! !

WAObject subclass: #WARouteResult
	instanceVariableNames: 'route elements parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

!WARouteResult class methodsFor: 'instance creation' stamp: 'pmm 9/8/2009 22:18'!
route: aRoute elements: anArray parameters: aDictionary
	^self basicNew initializeWithRoute: aRoute elements: anArray parameters: aDictionary! !

!WARouteResult methodsFor: 'accessing' stamp: 'pmm 9/2/2009 11:08'!
elements
	^ elements! !

!WARouteResult methodsFor: 'initialize-release' stamp: 'pmm 9/2/2009 11:10'!
initializeWithRoute: aRoute elements: anArray
	super initialize.
	route := aRoute.
	elements := anArray! !

!WARouteResult methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 22:18'!
initializeWithRoute: aRoute elements: anArray parameters: aDictionary
	super initialize.
	route := aRoute.
	elements := anArray.
	parameters := aDictionary! !

!WARouteResult methodsFor: 'accessing' stamp: 'pmm 9/2/2009 11:08'!
route
	^ route! !

!WARouteResult methodsFor: 'public' stamp: 'pmm 9/9/2009 19:31'!
sendMessageTo: anObject
	| arguments |
	arguments := Array new: route argumentCount.
	route mapArgumentsWithElements: elements parameters: parameters into: arguments.
	^ route
		sendSelectorWith: arguments
		to: anObject! !

WAError subclass: #WAArgumentNotFoundError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Route'!

Object subclass: #WAByMethodRoutesStub
	instanceVariableNames: 'selector arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WAByMethodRoutesStub methodsFor: 'accessing' stamp: 'pmm 9/9/2009 18:59'!
arguments
	^ arguments! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:01'!
compactDatabase: aString
	selector := #compactDatabase:.
	arguments := Array with: aString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:03'!
createBulkDocumentsInDatabase: aString
	selector := #createBulkDocumentsInDatabase:.
	arguments := Array with: aString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:01'!
createDatabase: aString
	selector := #createDatabase:.
	arguments := Array with: aString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:02'!
createNewDocumentInDatabase: aString
	selector := #createNewDocumentInDatabase:.
	arguments := Array with: aString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:55'!
createOrUpdateDocument: aFirstString inDatabase: aSecondString
	selector := #createOrUpdateDocument:inDatabase:.
	arguments := Array with: aFirstString with: aSecondString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:01'!
deleteDatabase: aString
	selector := #deleteDatabase:.
	arguments := Array with: aString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:03'!
deleteDocument: aFirstString inDatabase: aSecondString
	selector := #deleteDocument:inDatabase:.
	arguments := Array with: aFirstString with: aSecondString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:00'!
getAllDatabases
	selector := #getAllDatabases.
	arguments := Array new! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:02'!
getAllDocuments: aString
	selector := #getAllDocuments:.
	arguments := Array with: aString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:00'!
getConfig
	selector := #getConfig.
	arguments := Array new! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:02'!
getDatabase: aString
	selector := #getDatabase:.
	arguments := Array with: aString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:02'!
getDocument: aFirstString ofDatabase: aSecondString
	selector := #getDocument:ofDatabase:.
	arguments := Array with: aFirstString with: aSecondString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:00'!
getInfo
	selector := #getInfo.
	arguments := Array new! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:01'!
getStats
	selector := #getStats.
	arguments := Array new! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:01'!
getUuids: aString
	selector := #getUuids:.
	arguments := Array with: aString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:04'!
getView: aFirstString inDesign: aSecondString inDatabase: aThirdString
	selector := #getView:inDesign:inDatabase:.
	arguments := Array with: aFirstString with: aSecondString with: aThirdString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:03'!
queryDatabase: aString
	selector := #queryDatabase:.
	arguments := Array with: aString! !

!WAByMethodRoutesStub methodsFor: 'stubs' stamp: 'pmm 9/9/2009 19:01'!
replicate
	selector := #replicate.
	arguments := Array new! !

!WAByMethodRoutesStub methodsFor: 'accessing' stamp: 'pmm 9/9/2009 18:59'!
selector
	^ selector! !

Object subclass: #WARouteVictimMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Tests'!

!WARouteVictimMock methodsFor: 'accessing' stamp: 'pmm 9/2/2009 09:31'!
first: anObject
	^ Array with: anObject! !

!WARouteVictimMock methodsFor: 'accessing' stamp: 'pmm 9/2/2009 09:31'!
first: anFirstObject second: aSecondObject
	^ Array with: anFirstObject with: aSecondObject! !

!WARouteVictimMock methodsFor: 'accessing' stamp: 'pmm 9/2/2009 09:31'!
first: anFirstObject second: aSecondObject third: aThirdObject
	^ Array with: anFirstObject with: aSecondObject with: aThirdObject! !

!WARouteVictimMock methodsFor: 'accessing' stamp: 'pmm 9/2/2009 09:34'!
nothing
	^ Array new! !

WARequestFilter subclass: #WARestfulFilter
	instanceVariableNames: 'routes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core'!

WARestfulFilter subclass: #WAPragmaBasedRestfulFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Pharo'!

!WAPragmaBasedRestfulFilter methodsFor: 'private' stamp: 'pmm 6/27/2010 12:11'!
createRoutes
	^ WAPragmaBasedRouteBuilder createRoutesFor: self class! !

!WARestfulFilter methodsFor: 'private' stamp: 'pmm 6/27/2010 12:06'!
createRoutes
	self subclassResponsibility! !

!WARestfulFilter methodsFor: 'request-handling' stamp: 'DaleHenrichs 07/31/2010 08:31'!
handleFiltered: aRequestContext
	| result cp |
	cp := aRequestContext consumer path copy.
	result := self
		routeForMethod: aRequestContext request method
		consumer: aRequestContext consumer
		parameters: aRequestContext request url queryFields
		ifAbsent: [ 
			aRequestContext consumer initializeWith: cp.
			self noRouteFound: aRequestContext ].
	result sendMessageTo: self! !

!WARestfulFilter methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 22:56'!
initialize
	super initialize.
	self initializeRoutes! !

!WARestfulFilter methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 22:56'!
initializeRoutes
	routes := WAByMethodRoutes routes: self createRoutes! !

!WARestfulFilter methodsFor: 'private' stamp: 'pmm 6/27/2010 12:09'!
isImplemented: aString
	^ WARoute httpMethodNames includes: aString! !

!WARestfulFilter methodsFor: 'request-handling' stamp: 'pmm 6/27/2010 16:50'!
noRouteFound: aRequestContext
	aRequestContext responseGenerator
		notFound;
		respond! !

!WARestfulFilter methodsFor: 'private' stamp: 'pmm 6/27/2010 12:06'!
routeForMethod: aString consumer: aPathConsumer parameters: aDictionary ifAbsent: aNiliadicBlock
	^ routes
		routeForMethod: aString
		consumer: aPathConsumer
		parameters: aDictionary
		ifAbsent: aNiliadicBlock! !

!GRPackage class methodsFor: '*seaside-rest' stamp: 'pmm 5/8/2010 13:58'!
seasideRest
	^ self new
		name: 'Seaside-REST';
		description: 'Write RESTful web services using Seaside';
		url: #seasideAddonsUrl;
		yourself! !
